#include <avr/io.h>
#include <avr/pgmspace.h>
#include "hd44780.h"
#include <util/delay.h>
unsigned char zeile=0, spalte=0;

/**********************************************************************************
 *                                                                                *
 *             universelle HD44780 (und kompatibel) Ansteuerung                   *
 *                                                                                *
 *                               by Benedikt                                      *
 *                                                                                *
 *               Ansteuerung im 4bit Modus, mit und ohne RW Pin                   *
 *                                                                                *
 **********************************************************************************/

#define LCD_PORT   PORTA    // Hier eintragen an welchem Port D4-7 des LCDs angeschlossen sind
#define LCD_PORTI  PINA     // Hier eintragen an welchem Port D4-7 des LCDs angeschlossen sind, nur eben das PINx Register
#define LCD_DDR    DDRA     // Hier eintragen an welchem Port D4-7 des LCDs angeschlossen sind, nur eben das DDRx Register

#define COM_PORT   PORTD    // Hier eintragen an welchem Port RS, RW, E angeschlossen sind
#define COM_DDR    DDRD     // Hier eintragen an welchem Port RS, RW, E angeschlossen sind, nur eben das DDRx Register

#define LCD_D7      7       // Ist D7 vom LCD an PORTx.7 oder PORTx.3 angeschlossen (nur die beiden Werte sind erlaubt, die restlichen Pins mssen an 0,1,2 bzw. 4,5,6 angeschlossen sein)

#define RS          5       // Portbit an dem RS angeschlossen ist
#define RW         	6       // Portbit an dem RW angeschlossen ist (kann entfallen, wenn USE_RW auf 0 gesetzt wird)
#define ENABLE      7       // Portbit an dem E angeschlossen ist

#define USE_RW      1       // Hier eine 0 hinschreiben, wenn das LCD ohne RW Leitung verwendet wird, eine 1 wenn diese angeschlossen ist

#define ZEILEN      2       // Displayhhe in Zeilen
#define SPALTEN     16      // Displaybreite in Zeichen

void lcd_init(void)
{
#if (LCD_D7==3)
	LCD_DDR|= 15;
#elif (LCD_D7==7)
	LCD_DDR|= 240;
#else
  #error "Ungltige LCD Belegung !"
#endif

#if (USE_RW) 
    COM_DDR|= (1<<RS)|(1<<RW)|(1<<ENABLE);
    COM_PORT&=~(1<<RW);
#else
    COM_DDR|= (1<<RS)|(1<<ENABLE);
#endif
    COM_PORT&=~(1<<ENABLE);
    COM_PORT&=~(1<<RS);

    _delay_ms(10);
    _delay_ms(10);
    _delay_ms(10);
    _delay_ms(10);
    _delay_ms(10);
#if (LCD_D7==3)
    LCD_PORT=(LCD_PORT&240)|3;
#else
    LCD_PORT=(LCD_PORT&15)|48;     
#endif

    COM_PORT|=(1<<ENABLE);				// Initialize by instruction
    _delay_us(5);
    COM_PORT&=~(1<<ENABLE);
    _delay_ms(10);

    COM_PORT|=(1<<ENABLE);
    _delay_us(5);
    COM_PORT&=~(1<<ENABLE);
    _delay_ms(5);

    COM_PORT|=(1<<ENABLE);
    _delay_us(5);
    COM_PORT&=~(1<<ENABLE);
    _delay_ms(5);

#if (LCD_D7==3)
	LCD_PORT=(LCD_PORT&240)|2; 			// Set 4bit Interface
#else
	LCD_PORT=(LCD_PORT&15)|32;
#endif
    COM_PORT|=(1<<ENABLE);
    _delay_us(2);
    COM_PORT&=~(1<<ENABLE);
    _delay_ms(10);

#if (SPALTEN*ZEILEN>16)
    lcd_writecom (40);
#else
    lcd_writecom (40-8);
#endif
    lcd_writecom (12);
    lcd_writecom (6);
    lcd_writecom (1);
    _delay_ms(10);
}

void lcd_setadress(void)                // Set Adresspointer
{	if (zeile==0)
        lcd_writecom(128+spalte);
	else if (zeile==1)
        lcd_writecom(128+spalte+64);
	else if (zeile==2)
        lcd_writecom(128+spalte+16);
	else if (zeile==3)
        lcd_writecom(128+spalte+80);
}

#if (USE_RW)
void lcd_busy(void)
{	unsigned char temp;
    COM_PORT&=~(1<<RS);          
    COM_PORT|=(1<<RW);
#if (LCD_D7==3)
	LCD_DDR&=240;
#else
    LCD_DDR&=15;
#endif    
	do
	{   COM_PORT|=(1<<ENABLE);
		_delay_us(0.8);
		temp=LCD_PORTI;
        COM_PORT&=~(1<<ENABLE);
		_delay_us(1);
        COM_PORT|=(1<<ENABLE);
		_delay_us(0.8);
        COM_PORT&=~(1<<ENABLE);
	}
	while (temp&(1<<LCD_D7));
}
#endif

void lcd_write(unsigned char byte)
{	unsigned char temp;
#if (USE_RW)
    COM_PORT&=~(1<<RW);
#endif
#if (LCD_D7==3)
	LCD_DDR|=15;
	temp=LCD_PORT&240;

	LCD_PORT=temp|(byte/16);
    COM_PORT|=(1<<ENABLE);
	_delay_us(1);
    COM_PORT&=~(1<<ENABLE);
	_delay_us(1);

	LCD_PORT=temp|(byte&15);
    COM_PORT|=(1<<ENABLE);
	_delay_us(1);
    COM_PORT&=~(1<<ENABLE);
#else
	LCD_DDR|=240;
	temp=LCD_PORT&15;

    LCD_PORT=temp|(byte&240);

    COM_PORT|=(1<<ENABLE);
    _delay_us(1);
    COM_PORT&=~(1<<ENABLE);
	_delay_us(1);

    LCD_PORT=temp|((byte&15)*16);
    COM_PORT|=(1<<ENABLE);
    _delay_us(1);
    COM_PORT&=~(1<<ENABLE);
#endif
}

void lcd_writechar(unsigned char byte)                  // Write Databyte
{
	if (spalte>=SPALTEN) 
	{	spalte=0;
		zeile++;
		if (zeile>=ZEILEN) 
			zeile=0;
	}
	lcd_setadress();
	spalte++;
	lcd_writebyte(byte);
}

void lcd_writebyte(unsigned char byte)                  // Write Databyte
{    
#if (USE_RW)
	lcd_busy();
#endif
    COM_PORT|=(1<<RS);
	lcd_write(byte);
#if (!(USE_RW))
	_delay_us(100);
#endif
}

void lcd_writecom(unsigned char byte)                   // Write Command
{ 
#if (USE_RW)
	lcd_busy();
#endif
    COM_PORT&=~(1<<RS);
	lcd_write(byte);
#if (!(USE_RW))
	if ((byte==1)||(byte==2))        // Command 1 & 2 bentigen lngere Pause
		_delay_ms(4);
	else
		_delay_us(100);
#endif
}

void lcd_stringr (char *s)                              // Display String
{   while (*s) 
        lcd_writechar(*s++);
}

void lcd_string (const char *progmem_s)                 // Display String from flash
{   unsigned char c;
    while ((c = pgm_read_byte(progmem_s++)))
        lcd_writechar(c);
}
